<?php

namespace NoccyLabs\Chrononaut;

use Iterator;
use DateTime;

class DateRange implements Iterator
{
    /** @var Date */
    private $startDate;
    /** @var Date */
    private $endDate;
    /** @var int */
    private $numDays;
    /** @var Date */
    private $currentDay;
    /** @var int */
    private $currentIndex;

    /**
     *
     * @param Date $startDate
     */
    public function __construct(Date $startDate)
    {
        $this->startDate = $startDate;
    }

    /**
     *
     * @param Date $startDate
     * @param int $numDays
     */
    public static function createFromDays(Date $startDate, int $numDays)
    {
        $range = new DateRange($startDate);
        $range->setDays($numDays);
        return $range;
    }

    /**
     *
     * @param Date $startDate
     * @param Date $endDate
     */
    public static function createFromDates(Date $startDate, Date $endDate)
    {
        $range = new DateRange($startDate);
        $range->setEndDate($endDate);
        return $range;
    }

    public static function createForMonth(Date $date)
    {
        $first = (clone $date);
        $first->setDate($first->format("Y"), $first->format("m"), 1);
        $last = (clone $first);
        $last->modify("+1 month")->modify("-1 day");
        return self::createFromDates($first, $last);
    }

    public function setDays(int $numDays)
    {
        $this->numDays = $numDays;
        $this->endDate = (clone $this->startDate)->add(new \DateInterval("P" . ($numDays - 1) . "D"));
    }

    public function getDays(): int
    {
        return $this->numDays;
    }

    public function getStartDate(): Date
    {
        return $this->startDate;
    }

    public function getEndDate(): ?Date
    {
        return $this->endDate;
    }

    public function setEndDate(Date $endDate)
    {
        $this->endDate = $endDate;
        $this->numDays = $this->startDate->diff($this->endDate)->days + 1;
    }

    public function rewind()
    {
        $this->currentDay = clone $this->startDate;
        $this->currentIndex = 0;
    }

    public function current()
    {
        return $this->currentDay;
    }

    public function key()
    {
        return $this->currentIndex;
    }

    public function next()
    {
        $this->currentDay = $this->currentDay->getNextDay();
        $this->currentIndex++;
    }

    public function valid(): bool
    {
        return $this->currentDay <= $this->endDate;
    }

    public function getFormattedDates(string $format = "Y-m-d")
    {
        $dates = iterator_to_array($this);
        $formatted = array_map(function ($date) use ($format) {
            return $date->format($format);
        }, $dates);
        return $formatted;
    }
}
