<?php

namespace NoccyLabs\Chrononaut;

class DateRangeTest extends \PhpUnit\Framework\TestCase
{

    public function setup()
    { }

    public function teardown()
    { }

    public function testCreatingWithConstructor()
    {
        $date = new Date("2019-01-01");
        $range = new DateRange($date);
        $range->setDays(7);

        $this->assertEquals($date, $range->getStartDate());
        $this->assertEquals(7, $range->getDays());
    }

    public function testCreatingFromDateRange()
    {
        $start = new Date("2019-01-01");
        $end = new Date("2019-01-07");
        $range = DateRange::createFromDates($start, $end);

        $this->assertEquals($start, $range->getStartDate());
        $this->assertEquals($end, $range->getEndDate());
        $this->assertEquals(7, $range->getDays());
    }

    public function testCreatingFromDays()
    {
        $start = new Date("2019-01-01");
        $end = new Date("2019-01-07");
        $days = 7;
        $range = DateRange::createFromDays($start, $days);

        $this->assertEquals($start, $range->getStartDate());
        $this->assertEquals($end, $range->getEndDate());
        $this->assertEquals($days, $range->getDays());
    }

    public function testCreatingForMonth()
    {
        $start = new Date("2019-01-07");
        $first = new Date("2019-01-01");
        $last = new Date("2019-01-31");
        $days = 31;
        $range = DateRange::createForMonth($start);

        $this->assertEquals($first, $range->getStartDate());
        $this->assertEquals($last, $range->getEndDate());
        $this->assertEquals($days, $range->getDays());
    }

    public function testFormattingDates()
    {
        $date = new Date("2019-01-01");
        $range = new DateRange($date);
        $range->setDays(7);

        $expected = ['2019-01-01', '2019-01-02', '2019-01-03', '2019-01-04', '2019-01-05', '2019-01-06', '2019-01-07'];
        $dates = $range->getFormattedDates();
        $this->assertEquals($expected, $dates);

        $expected = ['20190101', '20190102', '20190103', '20190104', '20190105', '20190106', '20190107'];
        $dates = $range->getFormattedDates("Ymd");
        $this->assertEquals($expected, $dates);
    }
}
